# frozen_string_literal: true

require "yesql/statement"
require "yesql/version"
require "yesql/config/configuration"
require "yesql/query/performer"
require "yesql/errors/file_path_does_not_exist_error"
require "yesql/errors/no_bindings_provided_error"
require "yesql/errors/output_argument_error"

module ::YeSQL
  include ::YeSQL::Config

  BIND_REGEX = /(?<!:):(\w+)(?=\b)/

  def YeSQL(file_path, binds = {}, options = {})
    output = options[:output] || :rows

    validate(binds, file_path, output)
    execute(binds, file_path, output, options[:prepare])
  end

  private

  def validate(binds, file_path, output)
    ::YeSQL::Errors::FilePathDoesNotExistError.new(file_path).validate_file_path_existence
    ::YeSQL::Errors::NoBindingsProvidedError.new(binds, file_path).validate_statement_bindings
    ::YeSQL::Errors::OutputArgumentError.new(output).validate_output_options
  end

  def execute(binds, file_path, output, prepare)
    ::YeSQL::Query::Performer.new(
      bindings: binds,
      bind_statement: ::YeSQL::Statement.new(binds, file_path),
      file_path: file_path,
      output: output,
      prepare: prepare
    ).call
  end
end
