# frozen_string_literal: true

require "yesql/bindings/extract"

module ::YeSQL
  module Bindings
    class Extractor
      def initialize(bindings: {})
        @bindings = bindings
        @indexed_bindings = bindings.to_a
      end

      def call
        bindings
          .each_with_object({})
          .with_index(1) do |((key, value), hash), index|
            hash[key] = binding_extracts(hash, indexed_bindings, index, key, value)
          end
      end

      private

      attr_reader :bindings, :indexed_bindings

      def binding_extracts(hash, indexed_bindings, index, key, value)
        ::YeSQL::Bindings::Extract.new(hash, indexed_bindings, index, value).tap do |extract|
          break {
            bind: {
              vals: extract.bind_vals,
              vars: extract.bind_vars
            },
            last_val: extract.last_val,
            match: key,
            prev: extract.prev,
            value: value
          }
        end
      end
    end
  end
end
