# frozen_string_literal: true

module ::YeSQL
  module Errors
    class FilePathDoesNotExistError
      def initialize(file_path)
        @file_path = file_path
      end

      def validate_file_path_existence
        return if file_exists?

        raise ::NotImplementedError, message
      end

      private

      attr_reader :file_path

      def message
"\nSQL file \"#{file_path}\" does not exist in #{::YeSQL.config.path}.

Available SQL files are:

#{available_files}\n"
      end

      def file_exists?
        path_files.any? { |filename| filename.include?("#{file_path}.sql") }
      end

      def available_files
        path_files.map { |file| "- #{file}\n" }.join
      end

      def path_files
        @path_files ||= Dir["#{::YeSQL.config.path}/**/*.sql"]
      end
    end
  end
end
