# frozen_string_literal: true

require "yesql/utils/read"

module ::YeSQL
  module Errors
    class NoBindingsProvidedError
      include ::YeSQL::Utils::Read

      def initialize(binds, file_path)
        @binds = binds
        @file_path = file_path
      end

      def validate_statement_bindings
        return if statement_binds.size.zero?
        return if expected_binds?

        raise ::ArgumentError, message
      end

      private

      attr_reader :binds, :file_path

      def message
"\nYeSQL invoked without bindings.

Expected bindings are:

#{renderable_statement_binds}\n"
      end

      def statement_binds
        statement
          .scan(::YeSQL::BIND_REGEX)
          .tap { |scanned_binds| break (scanned_binds || []).sort }
      end

      def renderable_statement_binds
        statement_binds.flatten.map { |bind| "- `#{bind}`\n" }.join
      end

      def expected_binds?
        binds.is_a?(::Hash) && !binds.empty?
      end
    end
  end
end
