# frozen_string_literal: true

module ::YeSQL
  module Errors
    class OutputArgumentError
      def initialize(output)
        @output = output
      end

      def validate_output_options
        return if output.nil?
        return if OPTIONS.include?(output.to_sym)

        raise ArgumentError, message
      end

      private

      attr_reader :output

      OPTIONS = %i[columns hash rows].freeze
      private_constant :OPTIONS

      def message
"Unsupported `output` option given `#{output}`. Possible values are:
  - `columns`: returns an array with the columns from the result.
  - `hash`: returns an array of hashes combining both, the columns and rows from the statement result.
  - `rows`: returns an array of arrays for each row from the given SQL statement.\n"
      end
    end
  end
end
